/*
 *  (c) 2011 - 2014 Steffen Bondorf,
 *  
 *  				disco | Distributed Computer Systems Lab
 *                  University of Kaiserslautern, Germany
 *                                  
 *         All Rights Reserved.
 *
 *  This software is work in progress and is released in the hope that it will
 *  be useful to the scientific community. It is provided "as is" without
 *  express or implied warranty, including but not limited to the correctness
 *  of the code or its suitability for any particular purpose.
 *
 *  You are free to use this software for any non-commercial educational or
 *  research purpose, provided that this copyright notice is not removed or
 *  modified. For commercial uses please contact the respective author(s).
 *
 *  If you find our software useful, we would appreciate if you mentioned it
 *  in any publication arising from the use of this software or acknowledge
 *  our work otherwise. We would also like to hear of any fixes or useful
 *  extensions to this software.
 *
 */

package unikl.disco.demos;

import java.util.Iterator;
import java.util.LinkedList;

import unikl.disco.curves.ServiceCurve;
import unikl.disco.curves.ArrivalCurve;
import unikl.disco.nc.TotalFlowAnalysis;
import unikl.disco.nc.SeparateFlowAnalysis;
import unikl.disco.nc.PmooAnalysis;

import unikl.disco.network.Link;
import unikl.disco.network.Flow;
import unikl.disco.network.Network;
import unikl.disco.network.Server;

/**
 * 
 * @author Steffen Bondorf
 */
public class Demo4
{
	public Demo4() {}

	@SuppressWarnings("unused")
	public void run()
	{
		ServiceCurve service_curve = ServiceCurve.createRateLatency( 10.0e6, 0.01 );
		ServiceCurve max_service_curve = ServiceCurve.createRateLatency( 100.0e6, 0.001 );
		
		Network network = new Network();	

		try {
			int numServers = 9;
			Server[] servers = new Server[numServers];
			
			for ( int i = 0; i < numServers; i++ )
			{
				servers[i] = network.addServer( service_curve, max_service_curve );
				servers[i].setUseGamma( false );
				servers[i].setUseExtraGamma( false );
			}
		
			Link l_0_2 = network.addLink( servers[0], servers[2] );
			Link l_1_2 = network.addLink( servers[1], servers[2] );
			Link l_1_3 = network.addLink( servers[1], servers[3] );
			Link l_2_4 = network.addLink( servers[2], servers[4] );
			Link l_3_4 = network.addLink( servers[3], servers[4] );
			Link l_4_5 = network.addLink( servers[4], servers[5] );
			Link l_5_6 = network.addLink( servers[5], servers[6] );
			Link l_6_7 = network.addLink( servers[6], servers[7] );
			Link l_7_8 = network.addLink( servers[7], servers[8] );
				
			ArrivalCurve arrival_curve = ArrivalCurve.createTokenBucket( 0.1e6, 0.1 * 0.1e6 );
	
			LinkedList<Link> path0 = new LinkedList<Link>();
	
//			links need to be ordered from source server to sink server when defining a path manually
			path0.add(l_2_4);
			path0.add(l_4_5);
			path0.add(l_5_6);
			path0.add(l_6_7);
			path0.add(l_7_8);
	
			Flow f0 = network.addFlow( arrival_curve, path0 );
				
			LinkedList<Link> path1 = new LinkedList<Link>();
			path1.add(l_1_3);
			path1.add(l_3_4);
			path1.add(l_4_5);
			path1.add(l_5_6);
			
			Flow f1 = network.addFlow( arrival_curve, path1 );
			
			Iterator<Flow> flow_iter = network.getFlows().iterator();
			while ( flow_iter.hasNext() ) {
				Flow flow_of_interest = flow_iter.next();
			
				System.out.println( "Flow of interest : " + flow_of_interest.toString() );
				System.out.println();

//				Analyze the network	
//				TFA
				System.out.println( "--- Total Flow Analysis ---" );
				TotalFlowAnalysis tf = new TotalFlowAnalysis( network );
				
				try {
					tf.performEnd2EndAnalysis( flow_of_interest );
					System.out.println( "delay bound     : " + tf.getDelayBound() );
					System.out.println( "backlog bound   : " + tf.getBacklogBound() );
					System.out.println( "     per server : " + tf.getServerBacklogBoundMap().values().toString() );
				} catch (Exception e) {
					System.out.println( "TFA analysis failed" );
					System.out.println( e.toString() );
				}
				
				System.out.println();
		
//				SFA
				System.out.println( "--- Separated Flow Analysis ---" );
				SeparateFlowAnalysis sf = new SeparateFlowAnalysis( network );
		
				try {
					sf.performEnd2EndAnalysis( flow_of_interest );
					System.out.println( "e2e SFA SC      : " + sf.getLeftOverServiceCurve() );
					System.out.println( "     per server : " + sf.getServerLeftOverBetaMap().values().toString() );
					System.out.println( "delay bound     : " + sf.getDelayBound() );
					System.out.println( "backlog bound   : " + sf.getBacklogBound() );
				} catch (Exception e) {
					System.out.println( "SFA analysis failed" );
					System.out.println( e.toString() );
				}
				
				System.out.println();
		
//				PMOOSFA
				System.out.println( "--- PMOO Analysis ---" );
				PmooAnalysis pmoo = new PmooAnalysis( network );
				
				try {
					pmoo.performEnd2EndAnalysis( flow_of_interest );
					System.out.println( "e2e PMOO SFA SC : " + pmoo.getLeftOverServiceCurve() );
					System.out.println( "delay bound     : " + pmoo.getDelayBound() );
					System.out.println( "backlog bound   : " + pmoo.getBacklogBound() );
				} catch (Exception e) {
					System.out.println( "PMOO SFA analysis failed" );
					System.out.println( e.toString() );
				}

				System.out.println();
				System.out.println();
			}
		} catch (Exception e) {
			System.out.println( e.toString() );
		}
	}

	public static void main( String[] args )
	{
		Demo4 demo = new Demo4();
		demo.run();
	}
}
