/*
 *  (c) 2008 - 2014 Andreas Kiefer, Steffen Bondorf,
 *  
 *  				disco | Distributed Computer Systems Lab
 *                  University of Kaiserslautern, Germany
 *                                  
 *         All Rights Reserved.
 *
 *  This software is work in progress and is released in the hope that it will
 *  be useful to the scientific community. It is provided "as is" without
 *  express or implied warranty, including but not limited to the correctness
 *  of the code or its suitability for any particular purpose.
 *
 *  You are free to use this software for any non-commercial educational or
 *  research purpose, provided that this copyright notice is not removed or
 *  modified. For commercial uses please contact the respective author(s).
 *
 *  If you find our software useful, we would appreciate if you mentioned it
 *  in any publication arising from the use of this software or acknowledge
 *  our work otherwise. We would also like to hear of any fixes or useful
 *  extensions to this software.
 *
 */

package unikl.disco.nc;

import java.util.HashMap;
import java.util.Map;

import unikl.disco.curves.ServiceCurve;
import unikl.disco.curves.ArrivalCurve;
import unikl.disco.misc.Num;
import unikl.disco.network.Flow;
import unikl.disco.network.Network;
import unikl.disco.network.Path;
import unikl.disco.network.Server;

/**
 * This class contains all members and methods that are needed for more than one
 * analysis.
 * 
 * @author Frank A. Zdarsky
 * @author Andreas Kiefer
 * @author Steffen Bondorf
 */
public abstract class Analysis
{
	protected Network network = null;
	protected Flow flow_of_interest = null;

	protected Map<Server, ArrivalCurve> map__server__alpha = new HashMap<Server, ArrivalCurve>();
	
	protected ServiceCurve beta_e2e = ServiceCurve.createZeroDelayBurst();
	
	protected Num delay_bound = Num.ZERO;
	protected Num backlog_bound = Num.ZERO;

	public abstract void performEnd2EndAnalysis( Flow flow_of_interest ) throws Exception;
	
	public abstract void performAnalysisOnPath( Flow flow_of_interest, Path path ) throws Exception;

	public Network getNetwork() {
		return network;
	}

	public Flow getFlowOfInterest() {
		return flow_of_interest;
	}

	public ServiceCurve getLeftOverServiceCurve() {
		return beta_e2e;
	}
	
	/**
	 * Returns the delay bound of the analysis.
	 * 
	 * @return the delay bound
	 */
	public Num getDelayBound() {
		return delay_bound;
	};
	
	/**
	 * Returns the backlog bound of the analysis.
	 * 
	 * @return the backlog bound
	 */
	public Num getBacklogBound() {
		return backlog_bound;
	};

	/**
	 * For TFA this is the whole traffic at a server because
	 * you cannot separate the flow of interest from it anymore.
	 * 
	 * For SFA and PMOO you will get the arrival bounds of
	 * the cross-traffic at every server.
	 * 
	 * @return Mapping from the server to the server's arrival bound
	 */
	public Map<Server, ArrivalCurve> getServerAlphaMap(){
		return new HashMap<Server, ArrivalCurve>( map__server__alpha );
	}
	
	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString()
	{
		return getClass().getSimpleName();
	}
}
