/*
 *  (c) 2014 Steffen Bondorf,
 *  
 *                              disco | Distributed Computer Systems Lab
 *                  University of Kaiserslautern, Germany
 *                                  
 *         All Rights Reserved.
 *
 *  This software is work in progress and is released in the hope that it will
 *  be useful to the scientific community. It is provided "as is" without
 *  express or implied warranty, including but not limited to the correctness
 *  of the code or its suitability for any particular purpose.
 *
 *  You are free to use this software for any non-commercial educational or
 *  research purpose, provided that this copyright notice is not removed or
 *  modified. For commercial uses please contact the respective author(s).
 *
 *  If you find our software useful, we would appreciate if you mentioned it
 *  in any publication arising from the use of this software or acknowledge
 *  our work otherwise. We would also like to hear of any fixes or useful
 *  extensions to this software.
 *
 */

package unikl.disco.nc;

import java.util.ArrayList;

import unikl.disco.curves.Curve;
import unikl.disco.curves.ArrivalCurve;
import unikl.disco.curves.ServiceCurve;
import unikl.disco.misc.Num;

public class BacklogBound {
	public static Num derive( ArrivalCurve arrivals, ServiceCurve service ) {
		if ( service.equals( ServiceCurve.createZeroDelayBurst() ) ) {
			return Num.ZERO;
		}
		if ( arrivals.getSustainedRate().greater( service.getSustainedRate() ) ) {
			return Num.POSITIVE_INFINITY;
		}
		
		// The computeInflectionPoints based method does not work for 
		// single rate service curves (without latency)
		// in conjunction with token bucket arrival curves
		// because their common inflection point is in zero, 
		// where the arrival curve is 0.0 by definition.
		// This leads to a vertical deviation of 0 the arrival curve's burst
		// (or infinity which is already handled by the first if-statement)
		
		// Solution: 
		// Start with the burst as minimum vertical deviation
		
		Num result = arrivals.fLimitRight( Num.ZERO );
				
		ArrayList<Num> xcoords = Curve.computeInflectionPointsX( arrivals, service );	
		for( int i = 0; i < xcoords.size(); i++ ) {
			Num ip_x = ( (Num) xcoords.get( i ) );

			Num backlog = Num.sub( arrivals.f( ip_x ), service.f( ip_x ) );
			result = Num.max( result, backlog );
		}
		return result;
	}
}
