/*
 *  (c) 2013 - 2014 Steffen Bondorf,
 *  
 *  				disco | Distributed Computer Systems Lab
 *                  University of Kaiserslautern, Germany
 *                                  
 *         All Rights Reserved.
 *
 *  This software is work in progress and is released in the hope that it will
 *  be useful to the scientific community. It is provided "as is" without
 *  express or implied warranty, including but not limited to the correctness
 *  of the code or its suitability for any particular purpose.
 *
 *  You are free to use this software for any non-commercial educational or
 *  research purpose, provided that this copyright notice is not removed or
 *  modified. For commercial uses please contact the respective author(s).
 *
 *  If you find our software useful, we would appreciate if you mentioned it
 *  in any publication arising from the use of this software or acknowledge
 *  our work otherwise. We would also like to hear of any fixes or useful
 *  extensions to this software.
 *
 */

package unikl.disco.nc;

public final class Configuration {
	//----- used by Server.java -----------------------------------------------------------------------------------
	private static boolean use_short_alias = true;
	//-------------------------------------------------------------------------------------------------------------
	
	//----- used by Analysis.java ---------------------------------------------------------------------------------
	/**
	 * This variable only effects TotalFlowArrivalBound.java:
	 * Whether to use recursive arrival bound computation instead of iterative
	 * computation. Although using the iterative computation will lead to a
	 * faster runtime compared to the recursive implementation, it suffers from
	 * using a lot of memory that can lead to an <code>OutOfMemoryError</code>. 
	 */
	private static boolean use_recursive_arrivalbound_comp	= true;
	//-------------------------------------------------------------------------------------------------------------
	
	//----- used by Curve.java, Analysis.java and MinPlus.java ----------------------------------------------------
	private static boolean PERFORM_CHECKS = true; // Set to false to increase performance.
	
	private static boolean createLabels = false;
	//-------------------------------------------------------------------------------------------------------------
	
	//----- used by Analysis.java, PmooSfaAnalysis.java, SeparateFlowAnalysis.java and TotalFlowAnalysis.java -----
	public enum MuxDiscipline { SERVERLOCAL_FLAG, GLOBAL_ARBITRARY, GLOBAL_FIFO }
	//-------------------------------------------------------------------------------------------------------------
	
	//----- used by Analysis.java, PmooSfaAnalysis.java, SeparateFlowAnalysis.java and TotalFlowAnalysis.java -----
	public enum GammaFlag { SERVERLOCAL_FLAG, GLOBALLY_ON, GLOBALLY_OFF }
	//-------------------------------------------------------------------------------------------------------------

	//----- used by Analysis.java, SeparateFlowAnalysis.java and TotalFlowAnalysis.java ---------------------------
	private static MuxDiscipline multiplexing_discipline = MuxDiscipline.SERVERLOCAL_FLAG;
	//-------------------------------------------------------------------------------------------------------------

	//----- used by Analysis.java and PmooSfaAnalysis.java --------------------------------------------------------
	private static GammaFlag use_gamma = GammaFlag.SERVERLOCAL_FLAG;
	//-------------------------------------------------------------------------------------------------------------

	//----- used by Analysis.java and PmooSfaAnalysis.java --------------------------------------------------------
	private static GammaFlag use_extra_gamma = GammaFlag.SERVERLOCAL_FLAG;
	//-------------------------------------------------------------------------------------------------------------

	//----- used by SeparateFlowAnalysis.java ---------------------------------------------------------------------
	public enum ArrivalBoundFlags { PER_HOP_BOUNDING, PMOO_BOUNDING }
	//-------------------------------------------------------------------------------------------------------------

	//----- used by SeparateFlowAnalysis.java ---------------------------------------------------------------------
	private static ArrivalBoundFlags arrival_bound_calculation = ArrivalBoundFlags.PER_HOP_BOUNDING;
	//-------------------------------------------------------------------------------------------------------------

	//-------------------------------------------------------------------------------------------------------------
	private static boolean use_cache = false;
	//-------------------------------------------------------------------------------------------------------------
	
	public static boolean useShortAliases()
	{
		return use_short_alias;
	}

	public static void setUseShortAliases( boolean short_alias )
	{
		use_short_alias = short_alias;
	}	
	
	public static boolean useRecursiveArrivalBoundComputation()
	{
		return use_recursive_arrivalbound_comp;
	}

	public static void setUseRecursiveArrivalBoundComputation( boolean recursive )
	{
		use_recursive_arrivalbound_comp = recursive;
	}

	public static boolean performChecks()
	{
		return PERFORM_CHECKS;
	}

	public static void setPerformChecks( boolean perform_checks )
	{
		PERFORM_CHECKS = perform_checks;
	}

	public static boolean createLabels()
	{
		return createLabels;
	}
	
	public static void setCreateLabels( boolean create_labels )
	{
		createLabels = create_labels;
	}
	
	public static MuxDiscipline multiplexingDiscipline()
	{
		return multiplexing_discipline;
	}
	
	public static void setMultiplexingDiscipline( MuxDiscipline mux_discipline )
	{
		multiplexing_discipline = mux_discipline;
	}	
	
	public static GammaFlag useGamma()
	{
		return use_gamma;
	}
	
	public static void setUseGamma( GammaFlag use_gamma_flag )
	{
		use_gamma = use_gamma_flag;
	}
	
	public static GammaFlag useExtraGamma()
	{
		return use_extra_gamma;
	}
	
	public static void setUseExtraGamma( GammaFlag use_extra_gamma_flag )
	{
		use_extra_gamma = use_extra_gamma_flag;
	}
	
	public static ArrivalBoundFlags arrivalBoundCalculation()
	{
		return arrival_bound_calculation;
	}
	
	public static void setArrivalBoundCalculation( ArrivalBoundFlags arrival_bound_calculation_flag )
	{
		arrival_bound_calculation = arrival_bound_calculation_flag;
	}
	
	public static boolean useCache()
	{
		return use_cache;
	}
	
	public static void setUseCache( boolean use_cache_flag )
	{
		use_cache = use_cache_flag;
	}
}
