/*
 *  (c) 2014 Steffen Bondorf,
 *  
 *                              disco | Distributed Computer Systems Lab
 *                  University of Kaiserslautern, Germany
 *                                  
 *         All Rights Reserved.
 *
 *  This software is work in progress and is released in the hope that it will
 *  be useful to the scientific community. It is provided "as is" without
 *  express or implied warranty, including but not limited to the correctness
 *  of the code or its suitability for any particular purpose.
 *
 *  You are free to use this software for any non-commercial educational or
 *  research purpose, provided that this copyright notice is not removed or
 *  modified. For commercial uses please contact the respective author(s).
 *
 *  If you find our software useful, we would appreciate if you mentioned it
 *  in any publication arising from the use of this software or acknowledge
 *  our work otherwise. We would also like to hear of any fixes or useful
 *  extensions to this software.
 *
 */

package unikl.disco.nc;

import unikl.disco.curves.Curve;
import unikl.disco.curves.ArrivalCurve;
import unikl.disco.curves.ServiceCurve;
import unikl.disco.misc.Num;

public class DelayBound {
	public static Num deriveARB( ArrivalCurve arrivals, ServiceCurve service ) {
		return Curve.getXIntersection( arrivals, service );
	}

	// single flow to be bound, i.e., fifo per micro flow holds
	public static Num deriveFIFO( ArrivalCurve arrivals, ServiceCurve service ) {
		if ( arrivals.getSustainedRate().greater( service.getSustainedRate() ) ) {
			return Num.POSITIVE_INFINITY;
		}

		Num result = Num.NEGATIVE_INFINITY;
		for( int i = 0; i < arrivals.getSegmentCount(); i++ ) {
			Num ip_y = arrivals.getSegment( i ).y0;

			Num delay = Num.sub( service.f_inv( ip_y, true ), arrivals.f_inv( ip_y, false ) );
			result = Num.max( result, delay );
		}
		for( int i = 0; i < service.getSegmentCount(); i++ ) {
			Num ip_y = service.getSegment( i ).y0;

			Num delay = Num.sub( service.f_inv( ip_y, true ), arrivals.f_inv( ip_y, false ) );
			result = Num.max( result, delay );
		}
		
		return Num.max( Num.ZERO, result );
	}
}
