/*
 *  (c) 2008 - 2014 Andreas Kiefer, Steffen Bondorf,
 *  
 *  				disco | Distributed Computer Systems Lab
 *                  University of Kaiserslautern, Germany
 *                                  
 *         All Rights Reserved.
 *
 *  This software is work in progress and is released in the hope that it will
 *  be useful to the scientific community. It is provided "as is" without
 *  express or implied warranty, including but not limited to the correctness
 *  of the code or its suitability for any particular purpose.
 *
 *  You are free to use this software for any non-commercial educational or
 *  research purpose, provided that this copyright notice is not removed or
 *  modified. For commercial uses please contact the respective author(s).
 *
 *  If you find our software useful, we would appreciate if you mentioned it
 *  in any publication arising from the use of this software or acknowledge
 *  our work otherwise. We would also like to hear of any fixes or useful
 *  extensions to this software.
 *
 */

package unikl.disco.nc;

import java.util.Map;
import java.util.HashMap;

import unikl.disco.curves.ServiceCurve;
import unikl.disco.curves.ArrivalCurve;
import unikl.disco.misc.Num;
import unikl.disco.nc.Configuration.MuxDiscipline;
import unikl.disco.network.Flow;
import unikl.disco.network.Network;
import unikl.disco.network.Path;
import unikl.disco.network.Server;

/**
 * @author Frank A. Zdarsky
 * @author Andreas Kiefer
 * @author Steffen Bondorf
 */
public class TotalFlowAnalysis extends Analysis
{
	Map<Server, Num> map__server__D_server = new HashMap<Server, Num>();
	Map<Server, Num> map__server__B_server = new HashMap<Server, Num>();
	
	@SuppressWarnings("unused")
	private TotalFlowAnalysis(){}
	
	public TotalFlowAnalysis( Network network ) {
		super.network = network;
	}
	
	public void performEnd2EndAnalysis(	Flow flow_of_interest ) throws Exception {
		performAnalysisOnPath( flow_of_interest, flow_of_interest.getPath() );
	}

	public void performAnalysisOnPath( Flow flow_of_interest, Path path ) throws Exception	
	{
		super.flow_of_interest = flow_of_interest;
		delay_bound = Num.ZERO;

		for ( Server server : path.getServers() )
		{
			// Here's the difference to SFA:
			// TFA not only needs the arrivals of the flow of interest's cross traffic
			// but the arrival of all flows at the server, including the flow of interest
			// and thus calls computeArrivalBound with all flows present at the server
			ArrivalCurve alpha_server = ArrivalBound.computeArrivalBound( network, server, network.getFlows( server ) );

			// According to the call of computeOutputBound there's no left-over service curve calculation
			ServiceCurve beta_server = server.getServiceCurve();
			Num delay_bound_server;
			Num backlog_bound_server = BacklogBound.derive( alpha_server, beta_server );

			// Is this a single flow, i.e., does fifo per micro flow hold?
			boolean fifo_per_micro_flow = false;
			if ( network.getFlows( server ).size() == 1 ) {
				fifo_per_micro_flow = true;
			}
			
			if( Configuration.multiplexingDiscipline() == MuxDiscipline.GLOBAL_FIFO
				|| ( Configuration.multiplexingDiscipline() == MuxDiscipline.SERVERLOCAL_FLAG & server.useFifoMultiplexing() )
				|| fifo_per_micro_flow )
			{
				delay_bound_server = DelayBound.deriveFIFO( alpha_server, beta_server );	
			} else {
				delay_bound_server = DelayBound.deriveARB( alpha_server, beta_server );
			}

			delay_bound = Num.add( delay_bound, delay_bound_server );

			map__server__D_server.put( server, delay_bound_server );
			map__server__B_server.put( server, backlog_bound_server );
			map__server__alpha.put( server, alpha_server );
		}
		
		Num backlog_bound_candidate = Num.ZERO;
        for( Num value : map__server__B_server.values() ) {
			if( value.ge( backlog_bound_candidate ) ) {
				backlog_bound_candidate = value;
			}
		}
        if ( backlog_bound_candidate.greater( Num.ZERO ) ) {
        	backlog_bound = backlog_bound_candidate;
        }
	}
	
	public Map<Server, Num> getServerDelayBoundMap(){
		return new HashMap<Server, Num>( map__server__D_server );
	}
	
	public Map<Server, Num> getServerBacklogBoundMap(){
		return new HashMap<Server, Num>( map__server__B_server );
	}
}
