/*
 *  (c) 2013 - 2014 Steffen Bondorf,
 *  
 *  				disco | Distributed Computer Systems Lab
 *                  University of Kaiserslautern, Germany
 *                                  
 *         All Rights Reserved.
 *
 *  This software is work in progress and is released in the hope that it will
 *  be useful to the scientific community. It is provided "as is" without
 *  express or implied warranty, including but not limited to the correctness
 *  of the code or its suitability for any particular purpose.
 *
 *  You are free to use this software for any non-commercial educational or
 *  research purpose, provided that this copyright notice is not removed or
 *  modified. For commercial uses please contact the respective author(s).
 *
 *  If you find our software useful, we would appreciate if you mentioned it
 *  in any publication arising from the use of this software or acknowledge
 *  our work otherwise. We would also like to hear of any fixes or useful
 *  extensions to this software.
 *
 */

package unikl.disco.tests;

import org.junit.BeforeClass;
import org.junit.Test;

import static org.junit.Assert.assertEquals;
import unikl.disco.curves.ServiceCurve;
import unikl.disco.curves.ArrivalCurve;
import unikl.disco.misc.Num;
import unikl.disco.nc.Configuration;
import unikl.disco.nc.TotalFlowAnalysis;
import unikl.disco.nc.SeparateFlowAnalysis;
import unikl.disco.nc.PmooAnalysis;
import unikl.disco.nc.Configuration.ArrivalBoundFlags;
import unikl.disco.network.Flow;
import unikl.disco.network.Network;
import unikl.disco.network.Server;

public class Node_10Flows_10ACs
{
	static final ServiceCurve service_curve = ServiceCurve.createRateLatency( 10, 10 );
	static final ServiceCurve max_service_curve = ServiceCurve.createRateLatency( 15, 5 );
	
	static ArrivalCurve[] arrival_curves = new ArrivalCurve[10];
	
	static Network network;
	static Server s0;
	static Flow f0, f6;
	
	@BeforeClass
	static public void createNetwork()
	{
		network = new Network();
		
		s0 = network.addServer( service_curve, max_service_curve );
		s0.setUseGamma( false );
		s0.setUseExtraGamma( false );
	
		try {
			for( int i = 1; i <= 10; i++ ){
				arrival_curves[i-1] = ArrivalCurve.createTokenBucket( i*0.1, i );
				switch( i ){
					case 1: f0 = network.addFlow( arrival_curves[i-1], s0 );
							break;
					case 7: f6 = network.addFlow( arrival_curves[i-1], s0 );
							break;
					default: network.addFlow( arrival_curves[i-1], s0 );
							 break;
				}
			}
		} catch (Exception e) {
			System.out.println( e.toString() );
			return;
		}

		Configuration.setArrivalBoundCalculation( ArrivalBoundFlags.PER_HOP_BOUNDING );
	}
	
//--------------------Flow 0--------------------	
	@Test
	public void f0_tfa_fifoMux()
	{
//		TFA: results will be equal for all flows as they share the same path of servers, however, calculations differ due to distinct explicit input nodes
		System.out.println( "Flow of interest : " + f0.toString() );
		System.out.println();
		
		s0.setUseFifoMultiplexing( true ); // default = false
		System.out.println("--- Total Flow Analysis (FIFO_MUX)---");
		TotalFlowAnalysis tf = new TotalFlowAnalysis( network );
				
		try {
			tf.performEnd2EndAnalysis( f0 );
			System.out.println( "delay bound     : " + tf.getDelayBound() );
			System.out.println( "     per server : " + tf.getServerDelayBoundMap().values().toString() );
			System.out.println( "backlog bound   : " + tf.getBacklogBound() );
			System.out.println( "     per server : " + tf.getServerBacklogBoundMap().values().toString() );
			System.out.println( "alpha per server: " + tf.getServerAlphaMap().values().toString() );
			System.out.println();
		} catch (Exception e) {
			System.out.println( "TFA analysis failed" );
			System.out.println( e.toString() );
			System.out.println();
		}
		
		assertEquals("TFA FIFO delay", new Num( 15.5 ), tf.getDelayBound() );
		assertEquals("TFA FIFO backlog", new Num( 110 ), tf.getBacklogBound() );
	}
	
	@Test
	public void f0_tfa_arbMux()
	{
//		TFA: results will be equal for all flows as they share the same path of servers, however, calculations differ due to distinct explicit input nodes
		System.out.println( "Flow of interest : " + f0.toString() );
		System.out.println();

		s0.setUseArbitraryMultiplexing( true ); // s0.useArbitraryMultiplexing() != s0.useFifoMultiplexing()
		System.out.println("--- Total Flow Analysis (ARB_MUX)---");
		TotalFlowAnalysis tf = new TotalFlowAnalysis( network );
				
		try {
			tf.performEnd2EndAnalysis( f0 );
			System.out.println( "delay bound     : " + tf.getDelayBound() );
			System.out.println( "     per server : " + tf.getServerDelayBoundMap().values().toString() );
			System.out.println( "backlog bound   : " + tf.getBacklogBound() );
			System.out.println( "     per server : " + tf.getServerBacklogBoundMap().values().toString() );
			System.out.println( "alpha per server: " + tf.getServerAlphaMap().values().toString() );
			System.out.println();
		} catch (Exception e) {
			System.out.println( "TFA analysis failed" );
			System.out.println( e.toString() );
			System.out.println();
		}
		
		assertEquals("TFA ARB delay", new Num( 310, 9 ), tf.getDelayBound() );
		assertEquals("TFA ARB backlog", new Num( 110 ), tf.getBacklogBound() );
	}
	
	@Test
	public void f0_sfa_fifoMux()
	{
		System.out.println( "Flow of interest : " + f0.toString() );
		System.out.println();

		s0.setUseFifoMultiplexing( true );
		System.out.println( "--- Separated Flow Analysis (FIFO_MUX)---" );
		SeparateFlowAnalysis sf = new SeparateFlowAnalysis( network );

		try {
			sf.performEnd2EndAnalysis( f0 );
			System.out.println( "e2e SFA SC      : " + sf.getLeftOverServiceCurve() );
			System.out.println( "     per server : " + sf.getServerLeftOverBetaMap().values().toString() );
			System.out.println( "xtx per server  : " + sf.getServerAlphaMap().values().toString() );
			System.out.println( "delay bound     : " + sf.getDelayBound() );
			System.out.println( "backlog bound   : " + sf.getBacklogBound() );
			System.out.println();
		} catch (Exception e) {
			System.out.println( "SFA analysis failed" );
			System.out.println( e.toString() );
			System.out.println();
		}
		
		assertEquals("SFA FIFO delay", new Num( 1796, 115 ), sf.getDelayBound() );
		assertEquals("SFA FIFO backlog", new Num( 127, 50 ), sf.getBacklogBound() );
	}
	
	@Test
	public void f0_sfa_arbMux()
	{
		System.out.println( "Flow of interest : " + f0.toString() );
		System.out.println();

		s0.setUseArbitraryMultiplexing( true );
		System.out.println( "--- Separated Flow Analysis (ARB_MUX)---" );
		SeparateFlowAnalysis sf = new SeparateFlowAnalysis( network );

		try {
			sf.performEnd2EndAnalysis( f0 );
			System.out.println( "e2e SFA SC      : " + sf.getLeftOverServiceCurve() );
			System.out.println( "     per server : " + sf.getServerLeftOverBetaMap().values().toString() );
			System.out.println( "xtx per server  : " + sf.getServerAlphaMap().values().toString() );
			System.out.println( "delay bound     : " + sf.getDelayBound() );
			System.out.println( "backlog bound   : " + sf.getBacklogBound() );
			System.out.println();
		} catch (Exception e) {
			System.out.println( "SFA analysis failed" );
			System.out.println( e.toString() );
			System.out.println();
		}
		
		assertEquals("SFA ARB delay", new Num( 775, 23 ), sf.getDelayBound() );
		assertEquals("SFA ARB backlog", new Num( 100, 23 ), sf.getBacklogBound() );
	}
	
	@Test
	public void f0_pmoo_arbMux()
	{
		System.out.println( "Flow of interest : " + f0.toString() );
		System.out.println();

		s0.setUseArbitraryMultiplexing( true );
		System.out.println( "--- PMOO Analysis (ARB_MUX) ---" );
		PmooAnalysis pmoo = new PmooAnalysis( network );
		
		try {
			pmoo.performEnd2EndAnalysis( f0 );
			System.out.println( "e2e PMOO SFA SC : " + pmoo.getLeftOverServiceCurve() );
			System.out.println( "xtx per server  : " + pmoo.getServerAlphaMap().values().toString() );
			System.out.println( "delay bound     : " + pmoo.getDelayBound() );
			System.out.println( "backlog bound   : " + pmoo.getBacklogBound() );
			System.out.println();
		} catch (Exception e) {
			System.out.println( "PMOO SFA analysis failed" );
			System.out.println( e.toString() );
			System.out.println();
		}
		
		assertEquals("PMOO ARB delay", new Num( 775, 23 ), pmoo.getDelayBound() );
		assertEquals("PMOO ARB backlog", new Num( 100, 23 ), pmoo.getBacklogBound() );
	}

//--------------------Flow 6--------------------	
	@Test
	public void f6_tfa_fifoMux()
	{
//		TFA: results will be equal for all flows as they share the same path of servers, however, calculations differ due to distinct explicit input nodes
		System.out.println( "Flow of interest : " + f6.toString() );
		System.out.println();

		s0.setUseFifoMultiplexing( true ); // default = false
		System.out.println("--- Total Flow Analysis (FIFO_MUX)---");
		TotalFlowAnalysis tf = new TotalFlowAnalysis( network );
				
		try {
			tf.performEnd2EndAnalysis( f6 );
			System.out.println( "delay bound     : " + tf.getDelayBound() );
			System.out.println( "     per server : " + tf.getServerDelayBoundMap().values().toString() );
			System.out.println( "backlog bound   : " + tf.getBacklogBound() );
			System.out.println( "     per server : " + tf.getServerBacklogBoundMap().values().toString() );
			System.out.println( "alpha per server: " + tf.getServerAlphaMap().values().toString() );
			System.out.println();
		} catch (Exception e) {
			System.out.println( "TFA analysis failed" );
			System.out.println( e.toString() );
			System.out.println();
		}
		
		assertEquals("TFA FIFO delay", new Num( 15.5 ), tf.getDelayBound() );
		assertEquals("TFA FIFO backlog", new Num( 110 ), tf.getBacklogBound() );
	}
	
	@Test
	public void f6_tfa_arbMux()
	{
//		TFA: results will be equal for all flows as they share the same path of servers, however, calculations differ due to distinct explicit input nodes
		System.out.println( "Flow of interest : " + f6.toString() );
		System.out.println();

		s0.setUseArbitraryMultiplexing( true ); // s0.useArbitraryMultiplexing() != s0.useFifoMultiplexing()
		System.out.println("--- Total Flow Analysis (ARB_MUX)---");
		TotalFlowAnalysis tf = new TotalFlowAnalysis( network );
				
		try {
			tf.performEnd2EndAnalysis( f6 );
			System.out.println( "delay bound     : " + tf.getDelayBound() );
			System.out.println( "     per server : " + tf.getServerDelayBoundMap().values().toString() );
			System.out.println( "backlog bound   : " + tf.getBacklogBound() );
			System.out.println( "     per server : " + tf.getServerBacklogBoundMap().values().toString() );
			System.out.println( "alpha per server: " + tf.getServerAlphaMap().values().toString() );
			System.out.println();
		} catch (Exception e) {
			System.out.println( "TFA analysis failed" );
			System.out.println( e.toString() );
			System.out.println();
		}
		
		assertEquals("TFA ARB delay", new Num( 310, 9 ), tf.getDelayBound() );
		assertEquals("TFA ARB backlog", new Num( 110 ), tf.getBacklogBound() );;
	}
	
	@Test
	public void f6_sfa_fifoMux()
	{
		System.out.println( "Flow of interest : " + f6.toString() );
		System.out.println();

		s0.setUseFifoMultiplexing( true );
		System.out.println( "--- Separated Flow Analysis (FIFO_MUX)---" );
		SeparateFlowAnalysis sf = new SeparateFlowAnalysis( network );

		try {
			sf.performEnd2EndAnalysis( f6 );
			System.out.println( "e2e SFA SC      : " + sf.getLeftOverServiceCurve() );
			System.out.println( "     per server : " + sf.getServerLeftOverBetaMap().values().toString() );
			System.out.println( "xtx per server  : " + sf.getServerAlphaMap().values().toString() );
			System.out.println( "delay bound     : " + sf.getDelayBound() );
			System.out.println( "backlog bound   : " + sf.getBacklogBound() );
			System.out.println();
		} catch (Exception e) {
			System.out.println( "SFA analysis failed" );
			System.out.println( e.toString() );
			System.out.println();
		}

		assertEquals("SFA FIFO delay", new Num( 2099, 130 ), sf.getDelayBound() );
		assertEquals("SFA FIFO backlog", new Num( 434, 25 ), sf.getBacklogBound() );
	}
	
	@Test
	public void f6_sfa_arbMux()
	{
		System.out.println( "Flow of interest : " + f6.toString() );
		System.out.println();

		s0.setUseArbitraryMultiplexing( true );
		System.out.println( "--- Separated Flow Analysis (ARB_MUX)---" );
		SeparateFlowAnalysis sf = new SeparateFlowAnalysis( network );

		try {
			sf.performEnd2EndAnalysis( f6 );
			System.out.println( "e2e SFA SC      : " + sf.getLeftOverServiceCurve() );
			System.out.println( "     per server : " + sf.getServerLeftOverBetaMap().values().toString() );
			System.out.println( "xtx per server  : " + sf.getServerAlphaMap().values().toString() );
			System.out.println( "delay bound     : " + sf.getDelayBound() );
			System.out.println( "backlog bound   : " + sf.getBacklogBound() );
			System.out.println();
		} catch (Exception e) {
			System.out.println( "SFA analysis failed" );
			System.out.println( e.toString() );
			System.out.println();
		}

		assertEquals("SFA ARB delay", new Num( 775, 26 ), sf.getDelayBound() );
		assertEquals("SFA ARB backlog", new Num( 350, 13 ), sf.getBacklogBound() );
	}
	
	@Test
	public void f6_pmoo_arbMux()
	{
		System.out.println( "Flow of interest : " + f6.toString() );
		System.out.println();

		s0.setUseArbitraryMultiplexing( true );
		System.out.println( "--- PMOO Analysis (ARB_MUX) ---" );
		PmooAnalysis pmoo = new PmooAnalysis( network );
		
		try {
			pmoo.performEnd2EndAnalysis( f6 );
			System.out.println( "e2e PMOO SFA SC : " + pmoo.getLeftOverServiceCurve() );
			System.out.println( "xtx per server  : " + pmoo.getServerAlphaMap().values().toString() );
			System.out.println( "delay bound     : " + pmoo.getDelayBound() );
			System.out.println( "backlog bound   : " + pmoo.getBacklogBound() );
			System.out.println();
		} catch (Exception e) {
			System.out.println( "PMOO SFA analysis failed" );
			System.out.println( e.toString() );
			System.out.println();
		}
		
		assertEquals("PMOO ARB delay", new Num( 775, 26 ), pmoo.getDelayBound() );
		assertEquals("PMOO ARB backlog", new Num( 350, 13 ), pmoo.getBacklogBound() );
	}
}
