/*
 *  (c) 2013 - 2014 Steffen Bondorf,
 *  
 *  				disco | Distributed Computer Systems Lab
 *                  University of Kaiserslautern, Germany
 *                                  
 *         All Rights Reserved.
 *
 *  This software is work in progress and is released in the hope that it will
 *  be useful to the scientific community. It is provided "as is" without
 *  express or implied warranty, including but not limited to the correctness
 *  of the code or its suitability for any particular purpose.
 *
 *  You are free to use this software for any non-commercial educational or
 *  research purpose, provided that this copyright notice is not removed or
 *  modified. For commercial uses please contact the respective author(s).
 *
 *  If you find our software useful, we would appreciate if you mentioned it
 *  in any publication arising from the use of this software or acknowledge
 *  our work otherwise. We would also like to hear of any fixes or useful
 *  extensions to this software.
 *
 */

package unikl.disco.tests;

import static org.junit.Assert.assertEquals;

import org.junit.BeforeClass;
import org.junit.Test;

import unikl.disco.curves.ServiceCurve;
import unikl.disco.curves.ArrivalCurve;
import unikl.disco.misc.Num;
import unikl.disco.nc.Configuration;
import unikl.disco.nc.TotalFlowAnalysis;
import unikl.disco.nc.SeparateFlowAnalysis;
import unikl.disco.nc.PmooAnalysis;
import unikl.disco.nc.Configuration.ArrivalBoundFlags;
import unikl.disco.nc.Configuration.MuxDiscipline;
import unikl.disco.network.Flow;
import unikl.disco.network.Network;
import unikl.disco.network.Server;

public class Tandem_1SC_3Flows_1AC_3Paths
{
	static final ServiceCurve service_curve = ServiceCurve.createRateLatency( 20, 20 );
	static final ArrivalCurve arrival_curve = ArrivalCurve.createTokenBucket( 5, 25 );
	
	static Network network;
	static Server s0, s1, s2;
	static Flow f0, f1, f2;
	
	@BeforeClass
	static public void createNetwork()
	{
		network = new Network();
		s0 = network.addServer( service_curve );
		s1 = network.addServer( service_curve );
		s2 = network.addServer( service_curve );

		try {
			network.addLink( s0, s1 );
			network.addLink( s1, s2 );
		} catch (Exception e) {
			System.out.println( e.toString() );
			return;
		}

		try {	
			f0 = network.addFlow( arrival_curve, s0, s1 );
			f1 = network.addFlow( arrival_curve, s2 );
			f2 = network.addFlow( arrival_curve, s0, s2 );
		} catch (Exception e) {
			System.out.println( e.toString() );
			return;
		}
		
		Configuration.setArrivalBoundCalculation( ArrivalBoundFlags.PER_HOP_BOUNDING );
	}
	
//--------------------Flow 0--------------------	
	@Test
	public void f0_tfa_fifoMux()
	{
		System.out.println( "Flow of interest : " + f0.toString() );
		System.out.println();
		
		s0.setUseFifoMultiplexing( true ); // default = false
		s1.setUseFifoMultiplexing( true ); // default = false
		s2.setUseFifoMultiplexing( true ); // default = false
		
		System.out.println( "--- Total Flow Analysis (FIFO_MUX) ---" );
		TotalFlowAnalysis tf = new TotalFlowAnalysis( network );
		
		try {
			tf.performEnd2EndAnalysis( f0 );
			System.out.println( "delay bound     : " + tf.getDelayBound() );
			System.out.println( "     per server : " + tf.getServerDelayBoundMap().values().toString() );
			System.out.println( "backlog bound   : " + tf.getBacklogBound() );
			System.out.println( "     per server : " + tf.getServerBacklogBoundMap().values().toString() );
			System.out.println( "alpha per server: " + tf.getServerAlphaMap().values().toString() );
			System.out.println();
		} catch (Exception e) {
			System.out.println( "TFA analysis failed" );
			System.out.println( e.toString() );
			System.out.println();
		}
		
		assertEquals("TFA FIFO delay", new Num( 55 ), tf.getDelayBound() );
		assertEquals("TFA FIFO backlog", new Num( 450 ), tf.getBacklogBound() );
	}
	
	@Test
	public void f0_tfa_arbMux()
	{
		System.out.println( "Flow of interest : " + f0.toString() );
		System.out.println();
		
		s0.setUseArbitraryMultiplexing( true ); // default = true
		s1.setUseArbitraryMultiplexing( true ); // default = true
		s2.setUseArbitraryMultiplexing( true ); // default = true
		
		System.out.println( "--- Total Flow Analysis (ARB_MUX) ---" );
		TotalFlowAnalysis tf = new TotalFlowAnalysis( network );
		
		try {
			tf.performEnd2EndAnalysis( f0 );
			System.out.println( "delay bound     : " + tf.getDelayBound() );
			System.out.println( "     per server : " + tf.getServerDelayBoundMap().values().toString() );
			System.out.println( "backlog bound   : " + tf.getBacklogBound() );
			System.out.println( "     per server : " + tf.getServerBacklogBoundMap().values().toString() );
			System.out.println( "alpha per server: " + tf.getServerAlphaMap().values().toString() );
			System.out.println();
		} catch (Exception e) {
			System.out.println( "TFA analysis failed" );
			System.out.println( e.toString() );
			System.out.println();
		}

		assertEquals("TFA ARB delay", new Num( 110 ), tf.getDelayBound() );
		assertEquals("TFA ARB backlog", new Num( 450 ), tf.getBacklogBound() );
	}

	@Test
	public void f0_sfa_fifoMux()
	{
		System.out.println( "Flow of interest : " + f0.toString() );
		System.out.println();

		Configuration.setMultiplexingDiscipline( MuxDiscipline.GLOBAL_FIFO );
		
		System.out.println( "--- Separated Flow Analysis (FIFO_MUX) ---" );
		SeparateFlowAnalysis sf = new SeparateFlowAnalysis( network );

		try {
			sf.performEnd2EndAnalysis( f0 );
			System.out.println( "e2e SFA SC      : " + sf.getLeftOverServiceCurve() );
			System.out.println( "     per server : " + sf.getServerLeftOverBetaMap().values().toString() );
			System.out.println( "xtx per server  : " + sf.getServerAlphaMap().values().toString() );
			System.out.println( "delay bound     : " + sf.getDelayBound() );
			System.out.println( "backlog bound   : " + sf.getBacklogBound() );
			System.out.println();
		} catch (Exception e) {
			System.out.println( "SFA analysis failed" );
			System.out.println( e.toString() );
			System.out.println();
		}

		Configuration.setMultiplexingDiscipline( MuxDiscipline.SERVERLOCAL_FLAG );
		
		assertEquals("SFA FIFO delay", new Num( 295, 6 ), sf.getDelayBound() );
		assertEquals("SFA FIFO backlog", new Num( 262.5 ), sf.getBacklogBound() );
	}

	@Test
	public void f0_sfa_arbMux()
	{
		System.out.println( "Flow of interest : " + f0.toString() );
		System.out.println();
		
		s0.setUseArbitraryMultiplexing( true ); // default = true
		s1.setUseArbitraryMultiplexing( true ); // default = true
		s2.setUseArbitraryMultiplexing( true ); // default = true
		
		System.out.println( "--- Separated Flow Analysis (ARB_MUX) ---" );
		SeparateFlowAnalysis sf = new SeparateFlowAnalysis( network );

		try {
			sf.performEnd2EndAnalysis( f0 );
			System.out.println( "e2e SFA SC      : " + sf.getLeftOverServiceCurve() );
			System.out.println( "     per server : " + sf.getServerLeftOverBetaMap().values().toString() );
			System.out.println( "xtx per server  : " + sf.getServerAlphaMap().values().toString() );
			System.out.println( "delay bound     : " + sf.getDelayBound() );
			System.out.println( "backlog bound   : " + sf.getBacklogBound() );
			System.out.println();
		} catch (Exception e) {
			System.out.println( "SFA analysis failed" );
			System.out.println( e.toString() );
			System.out.println();
		}
		
		assertEquals("SFA ARB delay", new Num( 65 ), sf.getDelayBound() );
		assertEquals("SFA ARB backlog", new Num( 1025, 3 ), sf.getBacklogBound() );
	}
	
	@Test
	public void f0_pmoo_arbMux()
	{
		System.out.println( "Flow of interest : " + f0.toString() );
		System.out.println();
		
		s0.setUseArbitraryMultiplexing( true ); // default = true
		s1.setUseArbitraryMultiplexing( true ); // default = true
		s2.setUseArbitraryMultiplexing( true ); // default = true
		
		System.out.println( "--- PMOO Analysis (ARB_MUX) ---" );
		PmooAnalysis pmoo = new PmooAnalysis( network );
		
		try {
			pmoo.performEnd2EndAnalysis( f0 );
			System.out.println( "e2e PMOO SFA SC : " + pmoo.getLeftOverServiceCurve() );
			System.out.println( "xtx per server  : " + pmoo.getServerAlphaMap().values().toString() );
			System.out.println( "delay bound     : " + pmoo.getDelayBound() );
			System.out.println( "backlog bound   : " + pmoo.getBacklogBound() );
			System.out.println();
		} catch (Exception e) {
			System.out.println( "PMOO SFA analysis failed" );
			System.out.println( e.toString() );
			System.out.println();
		}

		assertEquals("PMOO ARB delay", new Num( 170, 3 ), pmoo.getDelayBound() );
		assertEquals("PMOO ARB backlog", new Num( 300 ), pmoo.getBacklogBound() );
	}

//--------------------Flow 1--------------------	
	@Test
	public void f1_tfa_fifoMux()
	{
		System.out.println( "Flow of interest : " + f1.toString() );
		System.out.println();
		
		s0.setUseFifoMultiplexing( true ); // default = false
		s1.setUseFifoMultiplexing( true ); // default = false
		s2.setUseFifoMultiplexing( true ); // default = false
		
		System.out.println( "--- Total Flow Analysis (FIFO_MUX) ---" );
		TotalFlowAnalysis tf = new TotalFlowAnalysis( network );
		
		try {
			tf.performEnd2EndAnalysis( f1 );
			System.out.println( "delay bound     : " + tf.getDelayBound() );
			System.out.println( "     per server : " + tf.getServerDelayBoundMap().values().toString() );
			System.out.println( "backlog bound   : " + tf.getBacklogBound() );
			System.out.println( "     per server : " + tf.getServerBacklogBoundMap().values().toString() );
			System.out.println( "alpha per server: " + tf.getServerAlphaMap().values().toString() );
			System.out.println();
		} catch (Exception e) {
			System.out.println( "TFA analysis failed" );
			System.out.println( e.toString() );
			System.out.println();
		}
		
		assertEquals("TFA FIFO delay", new Num( 2205, 64 ), tf.getDelayBound() );
		assertEquals("TFA FIFO backlog", new Num( 7825, 16 ), tf.getBacklogBound() );
	}
	
	@Test
	public void f1_tfa_arbMux()
	{
		System.out.println( "Flow of interest : " + f1.toString() );
		System.out.println();
		
		s0.setUseArbitraryMultiplexing( true ); // default = true
		s1.setUseArbitraryMultiplexing( true ); // default = true
		s2.setUseArbitraryMultiplexing( true ); // default = true
		
		System.out.println( "--- Total Flow Analysis (ARB_MUX) ---" );
		TotalFlowAnalysis tf = new TotalFlowAnalysis( network );
		
		try {
			tf.performEnd2EndAnalysis( f1 );
			System.out.println( "delay bound     : " + tf.getDelayBound() );
			System.out.println( "     per server : " + tf.getServerDelayBoundMap().values().toString() );
			System.out.println( "backlog bound   : " + tf.getBacklogBound() );
			System.out.println( "     per server : " + tf.getServerBacklogBoundMap().values().toString() );
			System.out.println( "alpha per server: " + tf.getServerAlphaMap().values().toString() );
			System.out.println();
		} catch (Exception e) {
			System.out.println( "TFA analysis failed" );
			System.out.println( e.toString() );
			System.out.println();
		}
		
		assertEquals("TFA ARB delay", new Num( 1405, 18 ), tf.getDelayBound() );
		assertEquals("TFA ARB backlog", new Num( 5225, 9 ), tf.getBacklogBound() );
	}
	
	@Test
	public void f1_sfa_fifoMux()
	{
		System.out.println( "Flow of interest : " + f1.toString() );
		System.out.println();
		
		Configuration.setMultiplexingDiscipline( MuxDiscipline.GLOBAL_FIFO );
		
		System.out.println( "--- Separated Flow Analysis (FIFO_MUX) ---" );
		SeparateFlowAnalysis sf = new SeparateFlowAnalysis( network );

		try {
			sf.performEnd2EndAnalysis( f1 );
			System.out.println( "e2e SFA SC      : " + sf.getLeftOverServiceCurve() );
			System.out.println( "     per server : " + sf.getServerLeftOverBetaMap().values().toString() );
			System.out.println( "xtx per server  : " + sf.getServerAlphaMap().values().toString() );
			System.out.println( "delay bound     : " + sf.getDelayBound() );
			System.out.println( "backlog bound   : " + sf.getBacklogBound() );
			System.out.println();
		} catch (Exception e) {
			System.out.println( "SFA analysis failed" );
			System.out.println( e.toString() );
			System.out.println();
		}

		Configuration.setMultiplexingDiscipline( MuxDiscipline.SERVERLOCAL_FLAG );
		
		assertEquals("SFA FIFO delay", new Num( 6695, 192 ), sf.getDelayBound() );
		assertEquals("SFA FIFO backlog", new Num( 12225, 64 ), sf.getBacklogBound() );
	}
	
	@Test
	public void f1_sfa_arbMux()
	{
		System.out.println( "Flow of interest : " + f1.toString() );
		System.out.println();
		
		s0.setUseArbitraryMultiplexing( true ); // default = true
		s1.setUseArbitraryMultiplexing( true ); // default = true
		s2.setUseArbitraryMultiplexing( true ); // default = true
		
		System.out.println( "--- Separated Flow Analysis (ARB_MUX) ---" );
		SeparateFlowAnalysis sf = new SeparateFlowAnalysis( network );
		
		try {
			sf.performEnd2EndAnalysis( f1 );
			System.out.println( "e2e SFA SC      : " + sf.getLeftOverServiceCurve() );
			System.out.println( "     per server : " + sf.getServerLeftOverBetaMap().values().toString() );
			System.out.println( "xtx per server  : " + sf.getServerAlphaMap().values().toString() );
			System.out.println( "delay bound     : " + sf.getDelayBound() );
			System.out.println( "backlog bound   : " + sf.getBacklogBound() );
			System.out.println();
		} catch (Exception e) {
			System.out.println( "SFA analysis failed" );
			System.out.println( e.toString() );
			System.out.println();
		}
		
		assertEquals("SFA ARB delay", new Num( 1405, 27 ), sf.getDelayBound() );
		assertEquals("SFA ARB backlog", new Num( 7475, 27 ), sf.getBacklogBound() );
	}
	
	@Test
	public void f1_pmoo_arbMux()
	{
		System.out.println( "Flow of interest : " + f1.toString() );
		System.out.println();
		
		s0.setUseArbitraryMultiplexing( true ); // default = true
		s1.setUseArbitraryMultiplexing( true ); // default = true
		s2.setUseArbitraryMultiplexing( true ); // default = true
		
		System.out.println( "--- PMOO Analysis (ARB_MUX) ---" );
		PmooAnalysis pmoo = new PmooAnalysis( network );
		
		try {
			pmoo.performEnd2EndAnalysis( f1 );
			System.out.println( "e2e PMOO SFA SC : " + pmoo.getLeftOverServiceCurve() );
			System.out.println( "xtx per server  : " + pmoo.getServerAlphaMap().values().toString() );
			System.out.println( "delay bound     : " + pmoo.getDelayBound() );
			System.out.println( "backlog bound   : " + pmoo.getBacklogBound() );
			System.out.println();
		} catch (Exception e) {
			System.out.println( "PMOO SFA analysis failed" );
			System.out.println( e.toString() );
			System.out.println();
		}
		
		assertEquals("PMOO ARB delay", new Num( 1405, 27 ), pmoo.getDelayBound() );
		assertEquals("PMOO ARB backlog", new Num( 7475, 27 ), pmoo.getBacklogBound() );
	}
//--------------------Flow 2--------------------	
	@Test
	public void f2_tfa_fifoMux()
	{
		System.out.println( "Flow of interest : " + f2.toString() );
		System.out.println();
		
		s0.setUseFifoMultiplexing( true ); // default = false
		s1.setUseFifoMultiplexing( true ); // default = false
		s2.setUseFifoMultiplexing( true ); // default = false
		
		System.out.println( "--- Total Flow Analysis (FIFO_MUX) ---" );
		TotalFlowAnalysis tf = new TotalFlowAnalysis( network );
		
		try {
			tf.performEnd2EndAnalysis( f2 );
			System.out.println( "delay bound     : " + tf.getDelayBound() );
			System.out.println( "     per server : " + tf.getServerDelayBoundMap().values().toString() );
			System.out.println( "backlog bound   : " + tf.getBacklogBound() );
			System.out.println( "     per server : " + tf.getServerBacklogBoundMap().values().toString() );
			System.out.println( "alpha per server: " + tf.getServerAlphaMap().values().toString() );
			System.out.println();
		} catch (Exception e) {
			System.out.println( "TFA analysis failed" );
			System.out.println( e.toString() );
			System.out.println();
		}
		
		assertEquals("TFA FIFO delay", new Num( 5725, 64 ), tf.getDelayBound() );
		assertEquals("TFA FIFO backlog", new Num( 7825, 16 ), tf.getBacklogBound() );
	}
	
	@Test
	public void f2_tfa_arbMux()
	{
		System.out.println( "Flow of interest : " + f2.toString() );
		System.out.println();
		
		s0.setUseArbitraryMultiplexing( true ); // default = true
		s1.setUseArbitraryMultiplexing( true ); // default = true
		s2.setUseArbitraryMultiplexing( true ); // default = true
		
		System.out.println( "--- Total Flow Analysis (ARB_MUX) ---" );
		TotalFlowAnalysis tf = new TotalFlowAnalysis( network );
		
		try {
			tf.performEnd2EndAnalysis( f2 );
			System.out.println( "delay bound     : " + tf.getDelayBound() );
			System.out.println( "     per server : " + tf.getServerDelayBoundMap().values().toString() );
			System.out.println( "backlog bound   : " + tf.getBacklogBound() );
			System.out.println( "     per server : " + tf.getServerBacklogBoundMap().values().toString() );
			System.out.println( "alpha per server: " + tf.getServerAlphaMap().values().toString() );
			System.out.println();
		} catch (Exception e) {
			System.out.println( "TFA analysis failed" );
			System.out.println( e.toString() );
			System.out.println();
		}
		
		assertEquals("TFA ARB delay", new Num( 16925, 90 ), tf.getDelayBound() );
		assertEquals("TFA ARB backlog", new Num( 5225, 9 ), tf.getBacklogBound() );
	}
	
	@Test
	public void f2_sfa_fifoMux()
	{
		System.out.println( "Flow of interest : " + f2.toString() );
		System.out.println();
		
		Configuration.setMultiplexingDiscipline( MuxDiscipline.GLOBAL_FIFO );
		
		System.out.println( "--- Separated Flow Analysis (FIFO_MUX) ---" );
		SeparateFlowAnalysis sf = new SeparateFlowAnalysis( network );

		try {
			sf.performEnd2EndAnalysis( f2 );
			System.out.println( "e2e SFA SC      : " + sf.getLeftOverServiceCurve() );
			System.out.println( "     per server : " + sf.getServerLeftOverBetaMap().values().toString() );
			System.out.println( "xtx per server  : " + sf.getServerAlphaMap().values().toString() );
			System.out.println( "delay bound     : " + sf.getDelayBound() );
			System.out.println( "backlog bound   : " + sf.getBacklogBound() );
			System.out.println();
		} catch (Exception e) {
			System.out.println( "SFA analysis failed" );
			System.out.println( e.toString() );
			System.out.println();
		}

		Configuration.setMultiplexingDiscipline( MuxDiscipline.SERVERLOCAL_FLAG );
		
		assertEquals("SFA FIFO delay", new Num( 845, 12 ), sf.getDelayBound() );
		assertEquals("SFA FIFO backlog", new Num( 1475, 4 ), sf.getBacklogBound() );
	}
	
	@Test
	public void f2_sfa_arbMux()
	{
		System.out.println( "Flow of interest : " + f2.toString() );
		System.out.println();
		
		s0.setUseArbitraryMultiplexing( true ); // default = true
		s1.setUseArbitraryMultiplexing( true ); // default = true
		s2.setUseArbitraryMultiplexing( true ); // default = true
		
		System.out.println( "--- Separated Flow Analysis (ARB_MUX) ---" );
		SeparateFlowAnalysis sf = new SeparateFlowAnalysis( network );

		try {
			sf.performEnd2EndAnalysis( f2 );
			System.out.println( "e2e SFA SC      : " + sf.getLeftOverServiceCurve() );
			System.out.println( "     per server : " + sf.getServerLeftOverBetaMap().values().toString() );
			System.out.println( "xtx per server  : " + sf.getServerAlphaMap().values().toString() );
			System.out.println( "delay bound     : " + sf.getDelayBound() );
			System.out.println( "backlog bound   : " + sf.getBacklogBound() );
			System.out.println();
		} catch (Exception e) {
			System.out.println( "SFA analysis failed" );
			System.out.println( e.toString() );
			System.out.println();
		}
		
		assertEquals("SFA ARB delay", new Num( 280, 3 ), sf.getDelayBound() );
		assertEquals("SFA ARB backlog", new Num( 1450, 3 ), sf.getBacklogBound() );
	}
	
	@Test
	public void f2_pmoo_arbMux()
	{
		System.out.println( "Flow of interest : " + f2.toString() );
		System.out.println();
		
		s0.setUseArbitraryMultiplexing( true ); // default = true
		s1.setUseArbitraryMultiplexing( true ); // default = true
		s2.setUseArbitraryMultiplexing( true ); // default = true
		
		System.out.println( "--- PMOO Analysis (ARB_MUX) ---" );
		PmooAnalysis pmoo = new PmooAnalysis( network );
		
		try {
			pmoo.performEnd2EndAnalysis( f2 );
			System.out.println( "e2e PMOO SFA SC : " + pmoo.getLeftOverServiceCurve() );
			System.out.println( "xtx per server  : " + pmoo.getServerAlphaMap().values().toString() );
			System.out.println( "delay bound     : " + pmoo.getDelayBound() );
			System.out.println( "backlog bound   : " + pmoo.getBacklogBound() );
			System.out.println();
		} catch (Exception e) {
			System.out.println( "PMOO SFA analysis failed" );
			System.out.println( e.toString() );
			System.out.println();
		}
		
		assertEquals("PMOO ARB delay", new Num( 85 ), pmoo.getDelayBound() );
		assertEquals("PMOO ARB backlog", new Num( 1325, 3 ), pmoo.getBacklogBound() );
	}
}